SHADER version 1
@OpenGL2.Vertex
#version 400
#define MAX_INSTANCES 256

//Variables 
float strength = 0.25;
float displacetex_size = 0.2;
float speed = 0.02;

//Uniforms
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform mat4 projectioncameramatrix;
uniform sampler2D texture3;
uniform float currenttime;
float time = currenttime/1000;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_tangent;
in vec3 vertex_binormal;

//Outputs
out vec4 ex_vertexposition;
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;

void main()
{
	//entity matrix
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_ = entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	//model position
	ex_vertexposition = entitymatrix_ * vec4(vertex_position,1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;

	//normals
	mat3 nmat = mat3(entitymatrix_);
	ex_normal = normalize(nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat * vertex_binormal);

	//texture coordinates
	ex_texcoords0 = vertex_texcoords0;
}
@OpenGLES2.Vertex

@OpenGLES2.Fragment

@OpenGL4.Vertex
#version 400
#define MAX_INSTANCES 256

//Variables 
float strength = 0.25;
float displacetex_size = 0.2;
float speed = 0.02;

//Uniforms
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform mat4 projectioncameramatrix;
uniform sampler2D texture3;
uniform float currenttime;
float time = currenttime/1000;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_tangent;
in vec3 vertex_binormal;

//Outputs
out vec4 ex_vertexposition;
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;

void main()
{
	//entity matrix
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_ = entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	//model position
	ex_vertexposition = entitymatrix_ * vec4(vertex_position,1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;

	//normals
	mat3 nmat = mat3(entitymatrix_);
	ex_normal = normalize(nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat * vertex_binormal);

	//texture coordinates
	ex_texcoords0 = vertex_texcoords0;
}
@OpenGL4.Fragment
//from https://www.shadertoy.com/view/MlKBWd

#version 400

//Variables
float heightnormal_bumpiness = 0.01;
float normal_mix = 0.5;

//Uniforms	
uniform sampler2D texture0;//diffuse
uniform sampler2D texture1;//normal
uniform sampler2D texture2;//specular
uniform sampler2D texture3;//height
uniform vec2 buffersize;

//Inputs
in vec4 ex_vertexposition;
in vec2 ex_texcoords0;
in vec3 ex_normal;
in vec3 ex_tangent;
in vec3 ex_binormal;

out vec4 fragData0;
out vec4 fragData1;
out vec4 fragData2;

void main() {

	//Diffuse 
	vec4 outcolor = texture(texture0, ex_texcoords0);
	
	//Specular 
	vec4 color_specular = texture(texture2,ex_texcoords0);

	//Normals from heightmap
	vec2 resolution = vec2(512);
    vec2 texelStep = 1.0 / resolution.xy;
    
	float tl = texture(texture3, ex_texcoords0 + vec2(-texelStep.x, -texelStep.y)).r;
	float l =  texture(texture3, ex_texcoords0 + vec2(-texelStep.x, 0)).r;
	float bl = texture(texture3, ex_texcoords0 + vec2(-texelStep.x, texelStep.y)).r;
	float t =  texture(texture3, ex_texcoords0 + vec2(0, -texelStep.y)).r;
	float b =  texture(texture3, ex_texcoords0 + vec2(0, texelStep.y)).r;
	float tr = texture(texture3, ex_texcoords0 + vec2(texelStep.x, -texelStep.y)).r;
	float r =  texture(texture3, ex_texcoords0 + vec2(texelStep.x, 0)).r;
	float br = texture(texture3, ex_texcoords0 + vec2(texelStep.x, texelStep.y)).r;
	
	float dX = tr + 2.0 * r + br - tl - 2.0 * l - bl;
	float dY = bl + 2.0 * b + br - tl - 2.0 * t - tr;
	
	//Heightnormal
	vec3 heightnormal = normalize(vec3(-dX * 255.0, -dY * 255.0, 1.0 / heightnormal_bumpiness));

	//Normal
	vec3 normal = ex_normal;
	normal = normalize(texture(texture1,ex_texcoords0).xyz * 2.0 - 1.0);
	normal = mix(normal,heightnormal,normal_mix);
	normal.z *= 2.0;
	normal = ex_tangent * normal.x + ex_binormal * normal.y + ex_normal * normal.z;	
	normal = normalize(normal);
	
	//Output
    fragData0 = outcolor * 0.7; 
	
	//Normals output
	fragData1 = vec4(normalize(normal) * 0.5 + 0.5, fragData0.a);
	fragData1.a = 1/255.0;
	
	//Specular Output
	float specular = color_specular.r * 0.299 + color_specular.g * 0.587 + color_specular.b * 0.114;
	fragData2 = vec4(0,0,0,specular);
}
